(*  title     : An encoding of ITL in Isabelle/HOL
    Authors   : Antonio Cau     <cau.researcher at gmail.com>
                Ben Moszkowski
                David Smallwood <drs at dmu.ac.uk>
    Maintainer: Antonio Cau     <cau.researcher at gmail.com>        
    License   : BSD
*)

theory ITL
imports
   Semantics
begin
                             
section  \<open>Axioms and Rules\<close>

text {*
The ITL axiom and proof rules are introduced (taken from 
\cite{moszkowski04:_hierar_compl_proof_propos_inter}) together with the validity operation.
The soundness of the rules and axioms are checked using the lemmas of Semantics.thy.
*}

definition valid :: "'a pitl \<Rightarrow> bool" ("(\<turnstile> _)" 5)
where "(\<turnstile> f) = (\<forall> \<sigma>. (\<sigma> \<Turnstile> f))"

lemma itl_valid [simp] :
  " (\<turnstile> f) = (\<forall> \<sigma>. (\<sigma> \<Turnstile> f))"
by (simp add: valid_def)

lemma itl_eq:
  " (\<turnstile> f \<equiv>\<^sub>i g) = (\<forall> \<sigma>. (\<sigma> \<Turnstile> f) = (\<sigma> \<Turnstile> g)) "
by simp  

lemma EqvReverseReverse:
 " \<turnstile>  (f\<^sup>r)\<^sup>r \<equiv>\<^sub>i f"
using TimeReverseSem by (metis interval_rev_rev_ident itl_eq)

lemma ReverseEqv:
 "  (\<turnstile> f) \<longleftrightarrow>(\<turnstile> f\<^sup>r) "
by (metis TimeReverseSem interval_rev_swap valid_def)

subsection \<open>Rules\<close>

lemma MP :
 assumes "\<turnstile> f \<supset>\<^sub>i g"
         "\<turnstile> f"
 shows   "\<turnstile> g"  
using assms(1) assms(2) by auto

lemma BoxGen :
 assumes "\<turnstile> f"
 shows   "\<turnstile> \<box> f"
using assms by auto
    
lemma BiGen:
 assumes "\<turnstile> f"
 shows   "\<turnstile> bi f"
using assms by auto

subsection \<open>Axioms\<close>
    
lemma ChopAssoc : 
    "\<turnstile>  f ; (g ; h) \<equiv>\<^sub>i  (f;g);h "
using  ChopAssocSem valid_def by blast
 
lemma OrChopImp :
    "\<turnstile> ( f \<or>\<^sub>i g);h   \<supset>\<^sub>i  f;h \<or>\<^sub>i g;h  "
using OrChopImpSem valid_def by blast

lemma ChopOrImp :
    "\<turnstile>  f;(g \<or>\<^sub>i h) \<supset>\<^sub>i   f;g \<or>\<^sub>i  f;h "
using ChopOrImpSem valid_def by blast
      
lemma EmptyChop :
    "\<turnstile>  empty ; f \<equiv>\<^sub>i f  "
using EmptyChopSem valid_def by blast

lemma ChopEmpty :
    "\<turnstile> f;empty \<equiv>\<^sub>i f  "
using ChopEmptySem valid_def by blast

lemma StateImpBi :
    "\<turnstile> init f \<supset>\<^sub>i   bi (init f)  "
using StateImpBiSem valid_def by blast

lemma NextImpNotNextNot :
    "\<turnstile>  \<circle> f \<supset>\<^sub>i  \<not>\<^sub>i (\<circle> \<not>\<^sub>i f) "
using NextImpNotNextNotSem valid_def by blast

lemma BiBoxChopImpChop :
    "\<turnstile>  bi ( f \<supset>\<^sub>i f1) \<and>\<^sub>i \<box>(g \<supset>\<^sub>i g1) \<supset>\<^sub>i f;g \<supset>\<^sub>i f1;g1 "
using BiBoxChopImpChopSem valid_def by blast

lemma BoxInduct :
    "\<turnstile> \<box> (f \<supset>\<^sub>i wnext f) \<and>\<^sub>i f \<supset>\<^sub>i \<box> f "
using BoxInductSem valid_def by blast
   
lemma ChopstarEqv :
    "\<turnstile>  f\<^sup>\<star> \<equiv>\<^sub>i  empty \<or>\<^sub>i  (f \<and>\<^sub>i more); f\<^sup>\<star> " 
using ChopstarEqvSem valid_def by blast

end